@@ -30,7 +30,7 @@
 
 from gi.repository import Gtk, GObject
 
-from gnomemusic.searchbar import Searchbar
+from gnomemusic.searchbar import Searchbar, DropDown
 
 from gnomemusic import log
 import logging
@@ -38,11 +38,9 @@ logger = logging.getLogger(__name__)
 
 
 class ToolbarState:
-    SINGLE = 0
-    ALBUMS = 1
-    ARTISTS = 2
-    PLAYLISTS = 3
-    SONGS = 4
+    MAIN = 0
+    CHILD_VIEW = 1
+    SEARCH_VIEW = 2
 
 
 class Toolbar(GObject.GObject):
@@ -56,13 +54,16 @@ class Toolbar(GObject.GObject):
     @log
     def __init__(self):
         GObject.GObject.__init__(self)
-        self._stack_switcher = Gtk.StackSwitcher(margin_top=2, margin_bottom=2, can_focus=False)
+        self._stack_switcher = Gtk.StackSwitcher(
+            margin_top=2, margin_bottom=2, can_focus=False, halign="center")
         self._stack_switcher.show()
         self._ui = Gtk.Builder()
         self._ui.add_from_resource('/org/gnome/Music/headerbar.ui')
         self.header_bar = self._ui.get_object('header-bar')
         self._search_button = self._ui.get_object('search-button')
-        self.searchbar = Searchbar(self._stack_switcher, self._search_button)
+        self.dropdown = DropDown()
+        self.searchbar = Searchbar(self._stack_switcher, self._search_button, self.dropdown)
+        self.dropdown.initialize_filters(self.searchbar)
         self._select_button = self._ui.get_object('select-button')
         self._cancel_button = self._ui.get_object('done-button')
         self._back_button = self._ui.get_object('back-button')
@@ -123,7 +124,7 @@ class Toolbar(GObject.GObject):
     def on_back_button_clicked(self, widget):
         view = self._stack_switcher.get_stack().get_visible_child()
         view._back_button_clicked(view)
-        self.set_state(ToolbarState.ALBUMS)
+        self.set_state(ToolbarState.MAIN)
 
     @log
     def set_state(self, state, btn=None):
@@ -133,12 +134,13 @@ class Toolbar(GObject.GObject):
 
     @log
     def _update(self):
-        if self._state == ToolbarState.SINGLE:
-            self.header_bar.set_custom_title(None)
-        elif self._selectionMode:
+        if self._selectionMode:
             self.header_bar.set_custom_title(self._selection_menu_button)
+        elif self._state != ToolbarState.MAIN:
+            self.header_bar.set_custom_title(None)
         else:
             self.reset_header_title()
 
-        self._back_button.set_visible(not self._selectionMode and self._state == ToolbarState.SINGLE)
+        self._search_button.set_visible(self._state != ToolbarState.SEARCH_VIEW)
+        self._back_button.set_visible(not self._selectionMode and self._state != ToolbarState.MAIN)
         self.header_bar.set_show_close_button(not self._selectionMode)
