@@ -32,7 +32,7 @@
 
 
 from gi.repository import Gtk, Gdk, Gd, GLib, GObject, Pango
-from gi.repository import GdkPixbuf, Gio, Grl
+from gi.repository import GdkPixbuf, Grl
 from gi.repository import Tracker
 from gettext import gettext as _, ngettext
 from gnomemusic.grilo import grilo
@@ -70,7 +70,6 @@ class AlbumWidget(Gtk.EventBox):
         super(Gtk.EventBox, self).__init__()
         self.player = player
         self.iterToClean = None
-        self.cache = AlbumArtCache.get_default()
 
         self.ui = Gtk.Builder()
         self.ui.add_from_resource('/org/gnome/Music/AlbumWidget.ui')
@@ -82,7 +81,6 @@ class AlbumWidget(Gtk.EventBox):
         self.album = None
         self.header_bar = None
         self.view.connect('item-activated', self._on_item_activated)
-        self.monitors = []
         view_box = self.ui.get_object('view')
         self.ui.get_object('scrolledWindow').set_placement(Gtk.CornerType.
                                                            TOP_LEFT)
@@ -263,10 +261,6 @@ class AlbumWidget(Gtk.EventBox):
             self.duration = self.duration + track.get_duration()
             _iter = self.model.append()
             self.player.discover_item(track, self._on_discovered, _iter)
-            g_file = Gio.file_new_for_uri(track.get_url())
-            self.monitors.append(g_file.monitor_file(Gio.FileMonitorFlags.NONE,
-                                                     None))
-            self.monitors[-1].connect('changed', self._on_item_changed, _iter)
             escapedTitle = AlbumArtCache.get_media_title(track, True)
             self.model.set(_iter,
                            [0, 1, 2, 3, 4, 5, 7, 9],
@@ -278,12 +272,6 @@ class AlbumWidget(Gtk.EventBox):
             self.ui.get_object('running_length_label_info').set_text(
                 '%d min' % (int(self.duration / 60) + 1))
 
-    @log
-    def _on_item_changed(self, monitor, file1, file2, event_type, _iter):
-        if self.model.iter_is_valid(_iter):
-            if event_type == Gio.FileMonitorEvent.DELETED:
-                self.model.set(_iter, [7, 9], [ERROR_ICON_NAME, True])
-
     @log
     def _on_look_up(self, pixbuf, path, data=None):
         _iter = self.iterToClean
@@ -433,6 +421,8 @@ class ArtistAlbums(Gtk.VBox):
 
     @log
     def set_selection_mode(self, selectionMode):
+        if self.selectionMode == selectionMode:
+            return
         self.selectionMode = selectionMode
         for widget in self.widgets:
             widget.set_selection_mode(selectionMode)
@@ -478,6 +468,8 @@ class AllArtistsAlbums(ArtistAlbums):
 
 class ArtistAlbumWidget(Gtk.HBox):
 
+    pixbuf = AlbumArtCache.get_default().get_default_icon(128, 128)
+
     @log
     def __init__(self, artist, album, player, model, header_bar, selectionModeAllowed):
         super(Gtk.HBox, self).__init__()
@@ -490,15 +482,12 @@ class ArtistAlbumWidget(Gtk.HBox):
         self.selectionMode = False
         self.selectionModeAllowed = selectionModeAllowed
         self.songs = []
-        self.monitors = []
         self.ui = Gtk.Builder()
         self.ui.add_from_resource('/org/gnome/Music/ArtistAlbumWidget.ui')
 
-        self.cache = AlbumArtCache.get_default()
-        pixbuf = self.cache.get_default_icon(128, 128)
         GLib.idle_add(self._update_album_art)
 
-        self.ui.get_object('cover').set_from_pixbuf(pixbuf)
+        self.ui.get_object('cover').set_from_pixbuf(self.pixbuf)
         self.ui.get_object('title').set_label(album.get_title())
         if album.get_creation_date():
             self.ui.get_object('year').set_markup(
@@ -552,10 +541,6 @@ class ArtistAlbumWidget(Gtk.HBox):
                     'toggled', self._check_button_toggled, song_widget
                 )
                 self.player.discover_item(track, self._on_discovered, song_widget)
-                g_file = Gio.file_new_for_uri(track.get_url())
-                self.monitors.append(g_file.monitor_file(Gio.FileMonitorFlags.NONE,
-                                                         None))
-                self.monitors[-1].connect('changed', self._on_item_changed, itr)
                 self.model.set(itr,
                                [0, 1, 2, 3, 4, 5],
                                [title, '', '', False,
@@ -571,12 +556,6 @@ class ArtistAlbumWidget(Gtk.HBox):
                                     self.track_selected)
             self.ui.get_object('grid1').show_all()
 
-    @log
-    def _on_item_changed(self, monitor, file1, file2, event_type, _iter):
-        if self.model.iter_is_valid(_iter):
-            if event_type == Gio.FileMonitorEvent.DELETED:
-                self.model.set(_iter, [3, 4], [True, ERROR_ICON_NAME])
-
     @log
     def _update_album_art(self):
         real_artist = self.album.get_string(Grl.METADATA_KEY_ARTIST)\
@@ -615,6 +594,8 @@ class ArtistAlbumWidget(Gtk.HBox):
 
     @log
     def set_selection_mode(self, selectionMode):
+        if self.selectionMode == selectionMode:
+            return
         self.selectionMode = selectionMode
         for songWidget in self.songs:
             songWidget.checkButton.set_visible(selectionMode)
@@ -628,6 +609,8 @@ class ArtistAlbumWidget(Gtk.HBox):
 
     @log
     def _model_row_changed(self, model, path, _iter):
+        if not self.selectionMode:
+            return
         if not model[_iter][5]:
             return
         songWidget = model[_iter][5].song_widget
@@ -650,18 +633,19 @@ class PlaylistDialog():
         self.view.connect('row-activated', self._on_item_activated)
 
         self.model = self.ui.get_object('liststore1')
-        playlist_names = playlist.get_playlists()
-        self.populate(playlist_names)
+        self.populate()
 
         self.title_bar = self.ui.get_object('headerbar1')
         self.dialog_box.set_titlebar(self.title_bar)
 
         self._cancel_button = self.ui.get_object('cancel-button')
         self._select_button = self.ui.get_object('select-button')
-        self._select_button.set_sensitive(len(playlist_names) > 0)
+        self._select_button.set_sensitive(False)
         self._cancel_button.connect('clicked', self._on_cancel_button_clicked)
         self._select_button.connect('clicked', self._on_selection)
 
+        playlist.connect('playlist-created', self._on_playlist_created)
+
     @log
     def get_selected(self):
         _iter = self.selection.get_selected()[1]
@@ -669,7 +653,7 @@ class PlaylistDialog():
         if not _iter or self.model[_iter][1]:
             return None
 
-        return self.model[_iter][0]
+        return self.model[_iter][2]
 
     @log
     def _add_list_renderers(self):
@@ -688,11 +672,26 @@ class PlaylistDialog():
         self.view.append_column(cols)
 
     @log
-    def populate(self, items):
-        for playlist_name in sorted(items):
-            self.model.append([playlist_name, False])
-        add_playlist_iter = self.model.append()
-        self.model.set(add_playlist_iter, [0, 1], [_("New Playlist"), True])
+    def populate(self):
+        self.add_playlist_iter = self.model.append()
+        self.model.set(self.add_playlist_iter, [0, 1], [_("New Playlist"), True])
+        if grilo.tracker:
+            GLib.idle_add(grilo.populate_playlists, 0, self._add_item)
+
+    @log
+    def _add_item(self, source, param, item, remaining=0, data=None):
+        if item:
+            self._add_item_to_model(item)
+
+    @log
+    def _add_item_to_model(self, item):
+        new_iter = self.model.insert_before(self.add_playlist_iter)
+        self.model.set(
+            new_iter,
+            [0, 1, 2],
+            [AlbumArtCache.get_media_title(item), False, item]
+        )
+        return new_iter
 
     @log
     def _on_list_text_render(self, col, cell, model, _iter, data):
@@ -726,12 +725,13 @@ class PlaylistDialog():
 
     @log
     def _on_editing_done(self, editable, data=None):
-        _iter = self.selection.get_selected()[1]
         if editable.get_text() != '':
             playlist.create_playlist(editable.get_text())
-            new_iter = self.model.insert_before(_iter)
-            self.model.set(new_iter, [0, 1], [editable.get_text(), False])
-            self.view.set_cursor(self.model.get_path(new_iter),
-                                 self.view.get_columns()[0], False)
-            self.view.row_activated(self.model.get_path(new_iter),
-                                    self.view.get_columns()[0])
+
+    @log
+    def _on_playlist_created(self, playlists, item):
+        new_iter = self._add_item_to_model(item)
+        self.view.set_cursor(self.model.get_path(new_iter),
+                             self.view.get_columns()[0], False)
+        self.view.row_activated(self.model.get_path(new_iter),
+                                self.view.get_columns()[0])
